"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../src/index");
const assert_1 = require("assert");
class Test {
    constructor(tests) {
        this._tests = tests;
    }
    run() {
        const CLEAR = "\x1b[0m";
        const BOLD = "\x1b[1m";
        const FAIL = "\x1b[31m";
        const OK = "\x1b[32m";
        for (let i = 0; i < this._tests.length; ++i) {
            process.stdout.write(`[${i + 1}/${this._tests.length}] - ${BOLD}`);
            try {
                this._tests[i]();
                process.stdout.write(`${OK}OK`);
            }
            catch (error) {
                process.stdout.write(`${FAIL}FAILED`);
            }
            finally {
                process.stdout.write(`${CLEAR}\n`);
            }
        }
    }
}
let test = new Test([
    () => {
        class Book extends index_1.JSONObject {
        }
        __decorate([
            index_1.JSONObject.required,
            __metadata("design:type", String)
        ], Book.prototype, "name", void 0);
        __decorate([
            index_1.JSONObject.optional,
            __metadata("design:type", String)
        ], Book.prototype, "summary", void 0);
        const title = 'Moby Dick';
        const summary = "You're my obsession";
        let book = new Book({ name: title, summary: summary });
        assert_1.strict.ok(book.name == title);
        assert_1.strict.ok(book.summary == summary);
        let b2 = new Book({ name: title }); // Also ok
        assert_1.strict.ok(book.name == title);
        assert_1.strict.ok(b2.summary === undefined);
        assert_1.strict.throws(() => {
            let book = new Book({ name: 12345 });
        });
        assert_1.strict.throws(() => {
            let book = new Book({ summary: "Once upon a time" });
        });
    },
    () => {
        class Author extends index_1.JSONObject {
        }
        __decorate([
            index_1.JSONObject.required,
            __metadata("design:type", String)
        ], Author.prototype, "name", void 0);
        class BookWithAuthor extends index_1.JSONObject {
        }
        __decorate([
            index_1.JSONObject.required,
            __metadata("design:type", String)
        ], BookWithAuthor.prototype, "name", void 0);
        __decorate([
            index_1.JSONObject.optional,
            __metadata("design:type", String)
        ], BookWithAuthor.prototype, "summary", void 0);
        __decorate([
            index_1.JSONObject.required,
            __metadata("design:type", Author)
        ], BookWithAuthor.prototype, "author", void 0);
        const title = 'Moby Dick';
        const author = "Herman Melville";
        let book = new BookWithAuthor({ name: title, author: { name: author } });
        assert_1.strict.ok(book.name == title);
        assert_1.strict.ok(book.author.name == author);
        assert_1.strict.ok(book.author instanceof Author);
    },
    () => {
        class Person extends index_1.JSONObject {
        }
        let p = new Person({ id: 10 });
        assert_1.strict.ok(p.id === undefined);
    },
    () => {
        class Person extends index_1.JSONObject {
        }
        __decorate([
            index_1.required,
            index_1.lt(10),
            index_1.gt(5),
            index_1.ne(6),
            __metadata("design:type", Number)
        ], Person.prototype, "id", void 0);
        assert_1.strict.throws(() => {
            let p = new Person({ id: 10 });
        });
        assert_1.strict.throws(() => {
            let p = new Person({ id: 5 });
        });
        assert_1.strict.throws(() => {
            let p = new Person({ id: 6 });
        });
        let p = new Person({ id: 7 });
    },
    () => {
        class Person extends index_1.JSONObject {
        }
        __decorate([
            index_1.required,
            index_1.eq(10),
            __metadata("design:type", Number)
        ], Person.prototype, "id", void 0);
        assert_1.strict.throws(() => {
            let p = new Person({ id: 11 });
        });
        let p = new Person({ id: 10 });
    },
    () => {
        class Person extends index_1.JSONObject {
        }
        __decorate([
            index_1.required,
            index_1.validate((person, key, value) => {
                if (value < 8)
                    throw new Error();
            }),
            __metadata("design:type", Number)
        ], Person.prototype, "id", void 0);
        let p = new Person({ id: 10 });
        assert_1.strict.throws(() => {
            let p = new Person({ id: 7 });
        });
    },
    () => {
        class Person extends index_1.JSONObject {
        }
        __decorate([
            index_1.required,
            index_1.union([1, 2, 3, 4, 5]),
            __metadata("design:type", Number)
        ], Person.prototype, "id", void 0);
        for (let i = 1; i <= 5; ++i) {
            let p = new Person({ id: i });
        }
        assert_1.strict.throws(() => {
            let p = new Person({ id: 6 });
        });
    },
    () => {
        const title = 'Moby Dick';
        class Book extends index_1.JSONObject {
        }
        __decorate([
            index_1.map('title'),
            index_1.required,
            __metadata("design:type", String)
        ], Book.prototype, "name", void 0);
        let book = new Book({ title: title });
        assert_1.strict.ok(book.name == title);
        assert_1.strict.throws(() => {
            let book = new Book({ name: title });
        });
    },
    () => {
        class Car extends index_1.JSONObject {
        }
        __decorate([
            index_1.passthrough,
            __metadata("design:type", Object)
        ], Car.prototype, "userManual", void 0);
        let car = new Car({});
        assert_1.strict.ok(car.userManual === undefined);
        let car2 = new Car({ userManual: '12345' });
        assert_1.strict.ok(car2.userManual == '12345');
        let car3 = new Car({ userManual: {
                pages: ['page1', 'page2']
            } });
        assert_1.strict.ok(car3.userManual.pages[1] == 'page2');
    }
]);
test.run();
// Summary is undefined
// This will throw a TypeError, as 'name' is required
//# sourceMappingURL=index.js.map